# -*- coding: utf-8 -*-
from results import OTCaching


class OT401Caching(OTCaching):
    def __init__(self, empowering_service, mongo_connection):
        super(OT401Caching, self).__init__(empowering_service, 'ot401_results',
                                  mongo_connection, 'ot401', 'empowering_error',
                                  'month', 'consumption')



# -*- coding: utf-8 -*-

from datetime import datetime
from dateutil.relativedelta import relativedelta

from results import OTCaching


class OT103Caching(OTCaching):
    def __init__(self, empowering_service, mongo_connection):
        super(OT103Caching, self).__init__(empowering_service, 'ot103_results',
                            mongo_connection, 'ot103', 'empowering_error',
                            'month', 'consumption')

    def get_cached(self, contract, period):
        month_range = 13
        end_period_date = datetime.strptime(period, '%Y%m')
        end_period_date = datetime(year=end_period_date.year,
                                   month=end_period_date.month,
                                   day=1)
        end_period = int(period)
        start_period_date = end_period_date + relativedelta(months=-(month_range))
        start_period = int(start_period_date.strftime('%Y%m'))

        query = {'contractId': contract,
                 'month': {'$gt': start_period,
                            '$lte': end_period}}
        res = self._result_collection.find(query).sort('month', 1)
        cached = [x for x in res]

        for elem in cached:
            for hidden_key in self._hidden_keys:
                if hidden_key in elem:
                    elem.pop(hidden_key)
        return {"_items": cached}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging
import os

import click

from amoniak import tasks
from amoniak.utils import setup_logging
from amoniak import VERSION


def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo(VERSION)
    ctx.exit()

@click.group()
@click.option('--log-level', default='info')
@click.option('--async/--no-async', default=True)
@click.option('--version', is_flag=True, callback=print_version,
              expose_value=False, is_eager=True)
def amoniak(log_level, async):
    MODE = {True: 'ASYNC', False: 'SYNC'}
    log_level = log_level.upper()
    log_level = getattr(logging, log_level, 'INFO')
    logging.basicConfig(level=log_level)
    setup_logging()
    logger = logging.getLogger('amon')
    logger.info('Running amoniak version: %s' % VERSION)
    logger.info('Running in %s mode' % MODE[async])
    os.environ['RQ_ASYNC'] = str(async)


@amoniak.command()
@click.option('--force', default=False, is_flag=True)
@click.argument('contracts', nargs=-1)
def enqueue_measures(contracts, force):
    logger = logging.getLogger('amon')
    force_log = force and '(F) ' or ''
    if contracts:
        logger.info('{}Enqueuing measures for contracts: {}'.format(force_log, ', '.join(contracts)))
    else:
        logger.info('{}Enqueuing all measures with etag'.format(force_log))
        contracts = None
    logger.info('Enqueuing measures')
    tasks.enqueue_measures(contracts=contracts, force=force)


@amoniak.command()
@click.option('--force', default=False, is_flag=True)
@click.argument('contracts', nargs=-1)
def enqueue_profiles(contracts, force):
    logger = logging.getLogger('amon')
    force_log = force and '(F) ' or ''
    if contracts:
        logger.info('{}Enqueuing profiles for contracts: {}'.format(force_log, ', '.join(contracts)))
    else:
        logger.info('{}Enqueuing all profiles with etag'.format(force_log))
        contracts = None
    logger.info('Enqueuing measures')
    tasks.enqueue_profiles(contracts=contracts, force=force)


@amoniak.command()
@click.argument('tariffs', nargs=-1)
def enqueue_tariffs(tariffs):
    logger = logging.getLogger('amon')
    if tariffs:
        logger.info(
            'Enqueuing tariffs: {}'.format(', '.join(tariffs)))
    else:
        logger.info('Enqueuing all tariffs')
        tariffs = None
    tasks.enqueue_tariffs(tariffs)


@amoniak.command()
@click.option('--force', default=False, is_flag=True)
def enqueue_new_contracts(force):
    logger = logging.getLogger('amon')
    logger.info('Enqueuing new contracts')
    tasks.enqueue_new_contracts(force=force)


@amoniak.command()
@click.option('--force', default=False, is_flag=True)
@click.argument('contracts', nargs=-1)
def enqueue_contract(contracts, force):
    logger = logging.getLogger('amon')
    force_log = force and '(F) ' or ''
    if contracts:
        logger.info('{}Enqueuing contracts: {}'.format(force_log, ', '.join(contracts)))
    else:
        logger.info('{}Enqueuing all contracts without etag'.format(force_log))
        contracts = None
    tasks.enqueue_contracts(contracts, force)

@amoniak.command()
@click.option('--force', default=False, is_flag=True)
@click.option('--pricelist', default=False)
@click.argument('wreport', nargs=-1)
def enqueue_indexed(force, pricelist, wreport):
    logger = logging.getLogger('amon')
    logger.info('Enqueuing indexed data')
    tasks.enqueue_indexed(force=force, pricelist=pricelist, wreport=wreport)

if __name__ == '__main__':
    amoniak(obj={})



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import
from hashlib import sha1
from copy import deepcopy
from datetime import datetime, timedelta
from pytz import timezone
import json
import logging
import os

from .cache import CUPS_CACHE, CUPS_UUIDS
from .utils import recursive_update, reduce_history, is_tertiary
from empowering.utils import remove_none, make_uuid, make_utc_timestamp


UNITS = {1: '', 1000: 'k'}

COLLECTION_UNITS = {
    'tg.cchfact': 'Wh',
    'tg.f1': 'kWh'
}

TZ = timezone('Europe/Madrid')

logger = logging.getLogger('amon')


def get_device_serial(device_id):
    return device_id[5:].lstrip('0')


def get_street_name(cups):
    street = []
    street_name = u''
    if cups['cpo'] or cups['cpa']:
        street = u'CPO %s CPA %s' % (cups['cpo'], cups['cpa'])
    else:
        if cups['tv']:
            street.append(cups['tv'][1])
        if cups['nv']:
            street.append(cups['nv'])
        street_name += u' '.join(street)
        street = [street_name]
        for f_name, f in [(u'número', 'pnp'), (u'escalera', 'es'),
                          (u'planta', 'pt'), (u'puerta', 'pu')]:
            val = cups.get(f, '')
            if val:
                street.append(u'%s %s' % (f_name, val))
    street_name = ', '.join(street)
    return street_name


def map_datetime(raw_timestamp):
    date, nhour = raw_timestamp.split(' ')
    current_date = TZ.localize(datetime.strptime(date, '%Y-%m-%d'))
    current_date = TZ.normalize(current_date + timedelta(hours=int(nhour)))
    return make_utc_timestamp(current_date)



class AmonConverter(object):
    def __init__(self, connection):
        self.O = connection

    def get_cups_from_device(self, serial):
        O = self.O
        # Remove brand prefix and right zeros
        if serial in CUPS_CACHE:
            return CUPS_CACHE[serial]
        else:
            # Search de meter
            cid = O.GiscedataLecturesComptador.search([
                ('name', '=', serial)
            ], context={'active_test': False})
            if not cid:
                res = False
            else:
                cid = O.GiscedataLecturesComptador.browse(cid[0])
                res = make_uuid('giscedata.cups.ps', cid.polissa.cups.name)
                CUPS_UUIDS[res] = cid.polissa.cups.id
                CUPS_CACHE[serial] = res
            return res

    def tariff_to_amon(self, pricelist_id, tariff_id):
        c = self.O
        tariff = c.GiscedataPolissaTarifa.read(tariff_id, ['name'])
        pricelist = c.ProductPricelist.browse(pricelist_id)
        uom_id = c.IrModelData.get_object_reference('giscedata_facturacio', 'uom_pot_elec_dia')[1]
        result = []
        for v in pricelist.version_id:
            date_start = v.date_start + ' 01:00:00'
            if v.date_end:
                date_end = (datetime.strptime(v.date_end, '%Y-%m-%d') + timedelta(days=1)).strftime('%Y-%m-%d')
            else:
                date_end = None
            tariff_cost_id = '{} ({})'.format(pricelist.name, pricelist.currency_id.name)
            tariff_name = tariff['name']
            price_date = date_start[:10]
            try:
                vals = {
                    'tariffCostId': tariff_cost_id,
                    'tariffId': tariff_name,
                    'dateStart': date_start and make_utc_timestamp(date_start),
                    'dateEnd': date_end and make_utc_timestamp(date_end),
                    'powerPrice': [round(v, 6) for k, v in sorted(c.GiscedataPolissaTarifa.get_periodes_preus(
                        tariff_id, 'tp', pricelist_id, {'date': price_date, 'uom': uom_id}
                    ).items())],
                    'energyPrice': [v for k, v in sorted(c.GiscedataPolissaTarifa.get_periodes_preus(
                        tariff_id, 'te', pricelist_id, {'date': price_date}
                    ).items())]
                }
                result.append(vals)
            except:
                logger.error(
                    "Error retrieving prices",
                    extra={'data': {
                        'pricelist': (pricelist_id, pricelist.name),
                        'tariff': (tariff_id, tariff_name),
                        'date': price_date
                    }}
                )
                continue
        return result

    def profiles_to_amon(self, profiles, collection='tg.cchfact'):
        c = self.O
        result = {}
        # TODO: We need a hack to convert meter serial to CUPS uuid
        # maybe we can have a global uuids cache pre-generated for that
        uuids = {}
        model = c.model(collection)
        for profile in model.read(profiles):
            cups = profile['name']
            if len(cups) != 22:
                cups = '{}0F'.format(cups)
            # Check if CUPS must be informed with 20 characters
            if int(os.getenv('CUPS_20_CHARACTERS', '0')):
                cups = cups[:20]
            m_point_id = uuids.get(cups)
            if not m_point_id:
                m_point_id = make_uuid('giscedata.cups.ps', cups)
                uuids[cups] = m_point_id
            result.setdefault(cups, {
                "measurements": [],
                "meteringPointId": m_point_id,
                "readings": [
                    {"type": "electricityConsumption", "period": "INSTANT",
                     "unit": COLLECTION_UNITS[collection]},
                ],
                "deviceId": m_point_id
            })
            result[cups]['measurements'] += [
                {
                    "timestamp": make_utc_timestamp(profile['datetime']),
                    "type": "electricityConsumption",
                    "value": profile['ai']
                }
            ]
        return result

    def aggregated_measures_to_amon(self, measures):
        res = {'R': [], 'T': []}

        for m in deepcopy(measures):
            values = {}
            cups = m['cups']
            # Check if CUPS must be informed with 20 characters
            if int(os.getenv('CUPS_20_CHARACTERS', '0')):
                cups = cups[:20]
            for agg in m['measures']:
                t = agg.pop('tipus')
                values.setdefault(t, {})
                values[t].update(agg)

            measurements = {
                'A': {
                    'timestamp': make_utc_timestamp(m['timestamp']),
                    'type': m['resource'] == 'R' and 'touElectricityConsumption' or 'tertiaryElectricityConsumption',
                    'values': values.get('A')
                },
                'R': {
                    'timestamp': make_utc_timestamp(m['timestamp']),
                    'type': m['resource'] == 'R' and 'touElectricityKiloVoltAmpHours' or 'tertiaryElectricityKiloVoltAmpHours',
                    'values': values.get('R')
                },
                'P': {
                    'timestamp': make_utc_timestamp(m['timestamp']),
                    'type': m['resource'] == 'R' and 'touPower' or 'tertiaryPower',
                    'values': values.get('P')
                }
            }
            deviceId = make_uuid('giscedata.cups.ps', cups)
            readings = []
            if measurements['A']['values']:
                readings.append({
                    "type": measurements['A']['type'],
                    "unit": "kWh",
                    "period": "INSTANT",
                })
            if measurements['R']['values']:
                readings.append({
                    "type": measurements['R']['type'],
                    "unit": "kVArh",
                    "period": "INSTANT",
                })
            if measurements['P']['values']:
                readings.append({
                    "type": measurements['P']['type'],
                    "unit": "kW",
                    "period": "INSTANT",
                })
            res[m['resource']].append({
                'deviceId': deviceId,
                'meteringPointId': deviceId,
                'readings': readings,
                'measurements': [v for v in measurements.values() if v['values']]
            })
        return res

    def power_measure_to_amon(self, measures):
        """Return a list of AMON readinds.

        {
            "utilityId": "Utility Id",
            "deviceId": "c1810810-0381-012d-25a8-0017f2cd3574",
            "meteringPointId": "c1759810-90f3-012e-0404-34159e211070",
            "readings": [
                {
                    "type_": "electricityConsumption",
                    "unit": "kWh",
                    "period": "INSTANT",
                },
                {
                    "type_": "electricityKiloVoltAmpHours",
                    "unit": "kVArh",
                    "period": "INSTANT",
                }
            ],
            "measurements": [
                {
                    "type_": "electricityConsumption",
                    "timestamp": "2010-07-02T11:39:09Z", # UTC
                    "value": 7
                },
                {
                    "type_": "electricityKiloVoltAmpHours",
                    "timestamp": "2010-07-02T11:44:09Z", # UTC
                    "value": 6
                }
            ]
        }
        """
        res = []
        if not hasattr(measures, '__iter__'):
            measures = [measures]

        for measure in measures:
            mp_uuid = make_uuid(
                'giscedata.cups.ps', measure.comptador.polissa.cups.name
            )
            device_uuid = make_uuid(
                'giscedata.lectures.comptador', measure.comptador.id
            )
            readings = []
            if measure.period.tarifa.name.startswith('2'):
                # measurements of 2.X
                readings += [{
                    "type":  "touPower",
                    "unit": "%sW" % UNITS[measure.get('magn', 1000)],
                    "period": "INSTANT",
                }]
            else:
                # tertiaryMeasurements
                readings += [{
                    "type": "tertiaryPower",
                    "unit": "%sW" % UNITS[measure.get('magn', 1000)],
                    "period": "INSTANT",
                }]

            res.append({
                "deviceId": device_uuid,
                "meteringPointId": mp_uuid,
                "readings": readings,
                "measurements": [
                    {
                        "type": readings[0]["type"],
                        "timestamp": make_utc_timestamp(measure.name),
                        "values": {
                            measure.periode.name: float(measure.lectura)
                        }
                    }
                ]
            })
        return res

    def energy_measure_to_amon(self, measures):
        """Return a list of AMON readinds.

        {
            "utilityId": "Utility Id",
            "deviceId": "c1810810-0381-012d-25a8-0017f2cd3574",
            "meteringPointId": "c1759810-90f3-012e-0404-34159e211070",
            "readings": [
                {
                    "type_": "electricityConsumption",
                    "unit": "kWh",
                    "period": "INSTANT",
                },
                {
                    "type_": "electricityKiloVoltAmpHours",
                    "unit": "kVArh",
                    "period": "INSTANT",
                }
            ],
            "measurements": [
                {
                    "type_": "electricityConsumption",
                    "timestamp": "2010-07-02T11:39:09Z", # UTC
                    "value": 7
                },
                {
                    "type_": "electricityKiloVoltAmpHours",
                    "timestamp": "2010-07-02T11:44:09Z", # UTC
                    "value": 6
                }
            ]
        }
        """
        res = []
        if not hasattr(measures, '__iter__'):
            measures = [measures]

        for measure in measures:
            mp_uuid = make_uuid(
                'giscedata.cups.ps', measure.comptador.polissa.cups.name
            )
            device_uuid = make_uuid(
                'giscedata.lectures.comptador', measure.comptador.id
            )
            readings = []
            if measure.period.tarifa.name.startswith('2'):
                # measurements of 2.X
                if measure.tipus == 'A':
                    readings += [{
                        "type":  "touElectricityConsumption",
                        "unit": "%sWh" % UNITS[measure.get('magn', 1000)],
                        "period": "INSTANT",
                    }]
                elif measure.tipus == 'R':
                    readings += [{
                        "type": "touElectricityKiloVoltAmpHours",
                        "unit": "%sVArh" % UNITS[measure.get('magn', 1000)],
                        "period": "INSTANT",
                    }]
            else:
                # tertiaryMeasurements
                if measure.tipus == 'A':
                    readings += [{
                        "type": "tertiaryElectricityConsumption",
                        "unit": "%sWh" % UNITS[measure.get('magn', 1000)],
                        "period": "INSTANT",
                    }]
                elif measure.tipus == 'R':
                    readings += [{
                        "type": "tertiaryElectricityKiloVoltAmpHours",
                        "unit": "%sVArh" % UNITS[measure.get('magn', 1000)],
                        "period": "INSTANT",
                    }]

            res.append({
                "deviceId": device_uuid,
                "meteringPointId": mp_uuid,
                "readings": readings,
                "measurements": [
                    {
                        "type": readings[0]["type"],
                        "timestamp": make_utc_timestamp(measure.name),
                        "value": float(measure.consum)
                    }
                ]
            })
        return res

    def contract_to_amon(self, contract_ids, context=None):
        """Converts contracts to AMON.

        {
          "payerId":"payerID-123",
          "ownerId":"ownerID-123",
          "signerId":"signerID-123",
          "power":123,
          "power_":{
            "p1":123,
            "p2":123,
            "dateStart":"2013-10-11T16:37:05Z",
            "dateEnd":null,
          }
          "dateStart":"2013-10-11T16:37:05Z",
          "dateEnd":null,
          "contractId":"contractID-123",
          "customer":{
            "customerId":"payerID-123",
            "address":{
              "city":"city-123",
              "cityCode":"cityCode-123",
              "countryCode":"ES",
              "country":"Spain",
              "street":"street-123",
              "postalCode":"postalCode-123"
            }
          },
          "meteringPointId":"c1759810-90f3-012e-0404-34159e211070",
          "devices":[
            {
              "dateStart":"2013-10-11T16:37:05Z",
              "dateEnd":null,
              "deviceId":"c1810810-0381-012d-25a8-0017f2cd3574"
            }
          ],
          "version":1,
          "activityCode":"activityCode",
          "tariffId":"tariffID-123",
        }
        """
        O = self.O
        if not context:
            context = {}
        res = []
        pol = O.GiscedataPolissa
        partner = O.ResPartner
        modcon_obj = O.GiscedataPolissaModcontractual
        if not hasattr(contract_ids, '__iter__'):
            contract_ids = [contract_ids]
        fields_to_read = [
            'modcontractual_activa', 'name', 'cups', 'comptadors', 'state',
            'tarifa', 'titular', 'pagador', 'data_alta', 'data_baixa',
            'llista_preu', 'cnae', 'modcontractuals_ids', 'potencia',
            'coeficient_d', 'coeficient_k', 'mode_facturacio', 'potencies_periode'
        ]
        for polissa in pol.read(contract_ids, fields_to_read):
            if polissa['state'] in ('esborrany', 'validar'):
                continue
            tarifa_atr = polissa['tarifa'][1]
            customer = partner.read(polissa['titular'][0], ['lang'])
            if polissa['mode_facturacio'] == 'index':
                fee = polissa['coeficient_d'] + polissa['coeficient_k']
                tariff_cost_id = '{} - {}'.format(polissa['llista_preu'][1], fee)
            else:
                tariff_cost_id = polissa['llista_preu'][1]
            cups = polissa['cups'][1]
            # Check if CUPS must be informed with 20 characters
            if int(os.getenv('CUPS_20_CHARACTERS', '0')):
                cups = cups[:20]
            contract = {
                'contractId': polissa['name'],
                'ownerId': make_uuid('res.partner', polissa['titular'][0]),
                'payerId': make_uuid('res.partner', polissa['pagador'][0]),
                'signerId': make_uuid('res.partner', polissa['pagador'][0]),
                'power': int(polissa['potencia'] * 1000),
                'dateStart': make_utc_timestamp(polissa['data_alta']),
                'dateEnd': make_utc_timestamp(polissa['data_baixa']),
                'tariffId': tarifa_atr,
                'tariffCostId': tariff_cost_id,
                'version': int(polissa['modcontractual_activa'][1]),
                'activityCode': polissa['cnae'] and polissa['cnae'][1].split(' ')[0] or None,
                'customer': {
                    'customerId': make_uuid('res.partner', polissa['titular'][0]),
                },
                'devices': self.device_to_amon(
                    polissa['comptadors'],
                    force_serial=make_uuid('giscedata.cups.ps', cups)
                ),
                'report': {
                    'language': customer['lang'] or 'ca_ES'
                }
            }
            # History fields
            history_fields = [
                ('tariffCostHistory', ['tariffCostId']),
                ('tariffHistory', ['tariffId']),
                ('powerHistory', ['power']),
                ('tertiaryPowerHistory', ['p1', 'p2', 'p3', 'p4', 'p5', 'p6'])
            ]
            for k, _ in history_fields:
                contract[k] = []
            modcon_fields = [
                'data_inici', 'data_final', 'llista_preu', 'tarifa', 'potencia',
                'mode_facturacio', 'coeficient_d', 'coeficient_k'
            ]
            mcon_activa = polissa['modcontractual_activa'][0]
            for modcon in O.GiscedataPolissaModcontractual.read(polissa['modcontractuals_ids'], modcon_fields):
                mod_tarifa_atr = modcon['tarifa'][1]

                if modcon['mode_facturacio'] == 'index':
                    fee = modcon['coeficient_d'] + modcon['coeficient_k']
                    tariff_cost_id = '{} - {}'.format(modcon['llista_preu'][1], fee)
                else:
                    tariff_cost_id = modcon['llista_preu'][1]

                contract['tariffCostHistory'].append({
                    'dateStart': make_utc_timestamp(modcon['data_inici']),
                    'dateEnd': make_utc_timestamp(modcon['data_final']),
                    'tariffCostId': tariff_cost_id
                })
                contract['tariffHistory'].append({
                    'dateStart': make_utc_timestamp(modcon['data_inici']),
                    'dateEnd': make_utc_timestamp(modcon['data_final']),
                    'tariffId': mod_tarifa_atr
                })

                # Fill tertiaryPowerHistory and powerHistory fields
                tertiary_power_history = {
                    'dateStart': make_utc_timestamp(modcon['data_inici']),
                    'dateEnd': make_utc_timestamp(modcon['data_final']),
                }
                for period, power in modcon_obj.get_potencies_dict(modcon['id']).items():
                    tertiary_power_history[period.lower()] = int(power * 1000)
                contract['tertiaryPowerHistory'].append(tertiary_power_history)

                power_history = {
                    'dateStart': make_utc_timestamp(modcon['data_inici']),
                    'dateEnd': make_utc_timestamp(modcon['data_final']),
                    'power': int(modcon['potencia'] * 1000)
                }
                contract['powerHistory'].append(power_history)

            # Reduce only for this changes
            for k, f in history_fields:
                contract[k] = reduce_history(contract[k], f)
                # Remove historic field if empty
                if not contract[k]:
                    contract.pop(k)

            # Get tertiary power
            contract['tertiaryPower'] = {}
            for period, power in pol.get_potencies_dict(polissa['id']).items():
                contract['tertiaryPower'][period.lower()] = int(power * 1000)
            contract['tertiaryPower_'] = contract['tertiaryPower'].copy()
            contract['tertiaryPower_'].update({'dateStart': make_utc_timestamp(polissa['data_alta'])})
            contract['tertiaryPower_'].update({'dateEnd': None})

            # Add custom fields
            customFields = pol.get_empowering_custom_fields(polissa['id'])
            if customFields:
                contract['customFields'] = customFields

            cups = self.cups_to_amon(polissa['cups'][0])
            recursive_update(contract, cups)
            res.append(contract)
        return res

    def device_to_amon(self, device_ids, force_serial=None):
        if not device_ids:
            return []
        compt_obj = self.O.GiscedataLecturesComptador
        devices = []
        comptador_fields = ['data_alta', 'data_baixa']
        for comptador in compt_obj.read(device_ids, comptador_fields):
            devices.append({
                'dateStart': make_utc_timestamp(comptador['data_alta']),
                'dateEnd': make_utc_timestamp(comptador['data_baixa']),
                'deviceId': force_serial or make_uuid('giscedata.lectures.comptador', comptador['id'])
            })
        return devices

    def cups_to_amon(self, cups_id):
        cups_obj = self.O.GiscedataCupsPs
        muni_obj = self.O.ResMunicipi
        cups_fields = ['id_municipi', 'tv', 'nv', 'cpa', 'cpo', 'pnp', 'pt',
                       'name', 'es', 'pu', 'dp']
        if 'empowering' in cups_obj.fields_get():
            cups_fields.append('empowering')
        cups = cups_obj.read(cups_id, cups_fields)
        cups_name = cups['name']
        # Check if CUPS must be informed with 20 characters
        if int(os.getenv('CUPS_20_CHARACTERS', '0')):
            cups_name = cups_name[:20]
        ine = muni_obj.read(cups['id_municipi'][0], ['ine'])['ine']
        res = {
            'meteringPointId': make_uuid('giscedata.cups.ps', cups_name),
            'customer': {
                'address': {
                    'city': cups['id_municipi'][1],
                    'cityCode': ine,
                    'countryCode': 'ES',
                    'street': get_street_name(cups),
                    'postalCode': cups['dp'] or None
                }
            },
            'experimentalGroupUserTest': False,
            'experimentalGroupUser': bool(cups.get('empowering', 0))
        }
        return res

    def indexed_to_amon(self, indexed_group, fact_ids):
        """
        indexed_group:  pricelist, cost
        fact_ids: list of invoice ids
        One grouped indexed to amon
        """
        # One grouped indexed to amon
        from base64 import b64decode
        import pandas as pd
        from StringIO import StringIO
        attach_obj = self.O.irAttachment
        tariff, tcost = indexed_group
        df_grouped = pd.DataFrame({})
        res = []
        for fact_id in fact_ids:
            att_id = attach_obj.search([
                ('res_model', '=', 'giscedata.facturacio.factura'),
                ('res_id', '=', fact_id),
                ('name', '=like', 'PH_%'),
            ])
            if not att_id:
                continue
            audit_data = attach_obj.read(att_id[0], ['datas'])['datas']
            audit_data = b64decode(audit_data)
            df = pd.read_csv(StringIO(audit_data), sep=';', names=['timestamp', 'price', 'raw', 'trash'])
            df = df[['timestamp', 'price']]
            if df_grouped.empty:
                df_grouped = df.copy()
            else:
                df_grouped = pd.concat([df_grouped, df])
        if df_grouped.empty:
            return res
        df_grouped = df_grouped.groupby('timestamp').median().reset_index()
        df_grouped['timestamp'] = df_grouped['timestamp'].apply(lambda x: map_datetime(x))
        for ts_indexed_median in df_grouped.T.to_dict().values():
            res.append({
                'tariffId': str(tariff),
                'tariffCostId': str(tcost),
                'price': ts_indexed_median['price'],
                'datetime': ts_indexed_median['timestamp'],
            })
        return res

def check_response(response, amon_data):
    logger.debug('Handlers: %s Class: %s' % (logger.handlers, logger))
    if response['_status'] != 'OK':
        content = '%s%s' % (json.dumps(amon_data), json.dumps(response))
        hash = sha1(content).hexdigest()[:8]
        logger.error(
            "Empowering response Code: %s - %s" % (response['_status'], hash),
            extra={'data': {
                'amon_data': amon_data,
                'response': response
            }}
        )
        return False
    return True



# -*- coding: utf-8 -*-

import time

AVALIABLE_ONLINE_TOOLS = ['ot101', 'ot201', 'ot103', 'ot503']

NO_RESULT_ERROR = 'No empowering result'
NO_STORED_ERROR = 'No stored value to compare with'
WRONG_VALUE_ERROR = 'Wrong value'

"""
" Useful class to cache the empowering results in a mongo database.
" Once pull_contract is called the mongo database will create a collection
" with the empowering resource name (db.ot101, db.ot103...)
"
" Also the cached results can be validated comapring with a dict
" with the correct values using the method validate_contract
" THis will delete the invalid cached results and fill the
" collection db.empowering_error with the errors found.
"
" Using the function error_report an string is generated
" using the db.empowering_error to generate a report.
"""


class OTCaching(object):

    def __init__(self, empowering_service, empowering_resource,
                 mongo_connection, ot_code, log_error_collection,
                 period_key, value_key):
        self._empowering_resource = getattr(empowering_service,
                                            empowering_resource)
        self._result_collection = getattr(mongo_connection, ot_code)
        self._log_error_collection = getattr(mongo_connection,
                                             log_error_collection)
        # Key used in results to specify the period
        self._period_key = period_key
        self._value_key = value_key
        self._ot_code = ot_code

        # Hidden keys not returned by get_cached
        self._hidden_keys = ['companyId', '_updated', '_etag',
                             '_id', '_created', '_links']

    def pull_contract(self, contract, period=None):
        """ Will ask for results of online tool for the specidied
        " contract for ALL periods if is not specidied in period param.
        " Pulled results will be stored in the mongo database
        "
        " If result already exist is replaced by the new one
        """
        results = self._empowering_resource().pull(contract=contract,
                                                   period=period)
        if '_items' not in results:
            # If _items not i results nothing to do, no results found
            return 0

        for result in results['_items']:
            result_period = result[self._period_key]
            if self._get(contract, result_period):
                # Delete cached result to replace it
                self._delete_cached(contract, result_period)
            self._store(result)

    def validate_contract(self, values, contract, period=None, log_errors=True):
        """ Validate the contract according to the values dict.
        " Values dict contain period as key and value as value.
        " Will create the error log in collection according to log_errors param
        "
        " values example:
        " {'201301': 42.2, '201302': 75.3}
        """
        cached_results = self._get(contract, period)
        for result in cached_results:
            cached_period = str(int(result[self._period_key]))
            cached_value = result[self._value_key]
            error = None
            error_details = {}

            if cached_period not in values:
                # No stored value to compare with.
                # So we discart the empowering result
                error = NO_STORED_ERROR
            elif not self._is_valid(cached_value, values[cached_period]):
                # Stored and empowering result missmatch
                # So we discart the empowering result
                error = WRONG_VALUE_ERROR
                error_details.update({
                    'expected': values[cached_period],
                    'cached': cached_value
                })
            else:
                pass #Everything OK :)

            if error:
                self._delete_cached(contract, cached_period)
                if log_errors:
                    self._insert_error(contract, cached_period, error,
                                       error_details)

            if cached_period in values:
                # Pop from values to know if there are missing results
                values.pop(cached_period)

        for v_period, v_value in values.iteritems():
            # There are still values to be checked
            error = NO_RESULT_ERROR
            self._insert_error(contract, v_period, error)

    def error_report(self, ot_code=None, contract=None, period=None,
                     validation_date='today'):
        """
        " @return an string containing a report of the errors happened
        """
        report = ''
        search_params = {}

        if validation_date == 'today':
            validation_date = time.strftime('%Y-%m-%d')

        if validation_date:
            search_params.update({'validation_date': validation_date})
        if period:
            search_params.update({'period': period})
        if contract:
            search_params.update({'contract': contract})

        old_search = {'validation_date': {'$ne': validation_date}}
        old_errors = self._log_error_collection.find(old_search).count()
        if old_errors:
            report += 'WARNING: There are %d stored old errors.\n' % old_errors

        filter_msg = 'REPORT FILTER: %s ot - %s contract %s period %s date\n'
        filter_msg %= (
            ot_code and ot_code or 'all',
            contract and contract or 'all',
            period and period or 'all',
            validation_date and validation_date or 'all'
        )
        report += filter_msg

        if not ot_code:
            ot_codes = AVALIABLE_ONLINE_TOOLS
        else:
            ot_codes = [ot_code]

        for ot in ot_codes:
            report += '%s\n' % ot
            for error in (WRONG_VALUE_ERROR, NO_RESULT_ERROR, NO_STORED_ERROR):
                errors = search_params.copy()
                errors.update({'error': error, 'ot_code': ot})
                count = self._log_error_collection.find(errors).count()
                report += '\t%s: %d\n' % (error, count)

        return report

    def error_clear(self, ot_code=None, contract=None, period=None,
                    validation_date='today'):
        """
        " Clear errors from database, si recomended to call this methond
        " just after error_report with the same parameters.
        """
        search_params = {}

        if validation_date == 'today':
            validation_date = time.strftime('%Y-%m-%d')

        if validation_date:
            search_params.update({'validation_date': validation_date})
        if ot_code:
            search_params.update({'ot_code': ot_code})
        if period:
            search_params.update({'period': period})
        if contract:
            search_params.update({'contract': contract})

        self._log_error_collection.remove(search_params)

    def get_cached(self, contract, period):
        cached = self._get(contract, period)
        for elem in cached:
            for hidden_key in self._hidden_keys:
                if hidden_key in elem:
                    elem.pop(hidden_key)
        return {"_items": cached}

    def _get(self, contract, period=None):
        query = {'contractId': contract}
        if period:
            query.update({self._period_key: int(period)})

        return [x for x in self._result_collection.find(query)]

    def _store(self, result):
        self._result_collection.insert(result)

    def _delete_cached(self, contract, period=None):
        remove_query = {'contractId': contract}
        if period:
            remove_query.update({
                self._period_key: int(period)
            })

        self._result_collection.remove(remove_query)

    def _is_valid(self, cached, reference):
        """ Return True if cached value is valid according to the
        " value reference, false otherwise
        """
        return abs(reference - cached) < 2

    def _insert_error(self, contract, period, error_message,
                      error_details=None):
        error = {
            'ot_code': self._ot_code,
            'contract': contract,
            'period': period,
            'error': error_message,
            'validation_date': time.strftime('%Y-%m-%d')
        }
        if error_details:
            error.update(error_details)
        if not self._log_error_collection.find(error).count():
            # Avoid insert errors twice
            self._log_error_collection.insert(error)




import os
import subprocess


try:
    try:
        VERSION = subprocess.check_output([
            'git', 'describe', '--tags'
        ]).strip()
    except subprocess.CalledProcessError as e:
        VERSION = __import__('pkg_resources') \
            .get_distribution(__name__).version
except Exception as e:
    VERSION = 'unknown'

from .amon import *



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from datetime import datetime
from dateutil.relativedelta import relativedelta
import logging
import urllib2

import libsaas

from .utils import (
    setup_peek, setup_empowering_api, setup_redis,
    sorted_by_key, Popper, setup_queue
)
from .amon import AmonConverter, check_response
from rq.decorators import job
from raven import Client
from empowering.utils import make_local_timestamp


sentry = Client()
logger = logging.getLogger('amon')


def enqueue_tariffs(tariffs=None):
    c = setup_peek()
    search_params = []
    if tariffs:
        search_params.append(('name', 'in', tariffs))
    tids = c.ProductPricelist.search(search_params, context={'active_test': False})
    to_q = []
    for pricelist in c.ProductPricelist.browse(tids):
        for tatr in pricelist.tarifes_atr_compatibles:
            n_polisses = c.GiscedataPolissa.search_count([
                ('tarifa.id', '=', tatr.id),
                ('llista_preu.id', '=', pricelist.id),
                ('etag', '!=', False)
            ], context={'active_test': False})
            t = (pricelist.id, tatr.id)
            if n_polisses and t not in to_q:
                logger.info('Enqueuing %s - %s', pricelist.name, tatr.name)
                to_q.append(t)
                push_tariffs.delay(t)


def enqueue_profiles(bucket=500, contracts=None, force=False):
    # First get all the contracts that are in sync
    c = setup_peek()
    # TODO: Que fem amb les de baixa? les agafem igualment? només les que
    # TODO: faci menys de X que estan donades de baixa?
    search_params = [('etag', '!=', False)]
    if contracts:
        search_params.append(('name', 'in', contracts))
    pids = c.GiscedataPolissa.search(search_params, context={'active_test': False})
    fields_to_read = ['name', 'cups', 'empowering_last_profile_measure', 'data_alta', 'data_baixa']
    from tqdm import tqdm
    for polissa in tqdm(c.GiscedataPolissa.read(pids, fields_to_read)):
        last_measure = polissa.get('empowering_last_profile_measure')
        cups = polissa['cups'][1]
        if not last_measure or force:
            logger.info("Les pugem totes")
            from_date = (
                datetime.now() - relativedelta(years=3)
            ).strftime('%Y-%m-%d')
            from_date = max(polissa['data_alta'], from_date)
            from_date = '{} 01:00:00'.format(from_date)
        else:
            from_date = last_measure
        if polissa['data_baixa']:
            to_date = (
                datetime.strptime(polissa['data_baixa'], '%Y-%m-%d') + relativedelta(days=1)
            ).strftime('%Y-%m-%d 00:00:00')
        else:
            to_date = ''
        logger.info(u"Pujant des de: %s fins a %s", from_date, to_date)
        # Use TM also
        for collection in ['tg.cchfact', 'tg.f1']:
            model = c.model(collection)
            cups_names = model.get_curve_cups(cups)
            search_profiles = [
                ('name', 'in', cups_names),
                ('datetime', '>', from_date)
            ]
            if polissa['data_baixa']:
                search_profiles += [
                    ('datetime', '<=', to_date)
                ]
            measures = model.search(search_profiles)
            logger.info("S'han trobat %s mesures (%s) per pujar", 
                len(measures), collection
            )
            popper = Popper(measures)
            pops = popper.pop(bucket)
            while pops:
                j = push_amon_profiles.delay(pops, collection)
                logger.info("Job id:%s | %s/%s/%s" % (
                    j.id, polissa['name'], len(pops), len(popper.items))
                )
                pops = popper.pop(bucket)


def enqueue_measures(bucket=500, contracts=None, force=False):
    # First get all the contracts that are in sync
    c = setup_peek()
    search_params = [('etag', '!=', False)]
    if contracts:
        search_params.append(('name', 'in', contracts))
    pids = c.GiscedataPolissa.search(search_params, context={'active_test': False})
    # Comptadors que tingui aquesta pòlissa i que siguin de telegestió
    cids = c.GiscedataLecturesComptador.search([
        ('polissa', 'in', pids)
    ], context={'active_test': False})
    fields_to_read = ['name', 'empowering_last_measure']
    for comptador in c.GiscedataLecturesComptador.read(cids, fields_to_read):
        last_measure = comptador.get('empowering_last_measure')
        if not last_measure or force:
            # Pujar totes
            logger.info("Les pugem totes")
            from_date = (
                datetime.now() - relativedelta(years=1)
            ).strftime('%Y-%m-%d')
            logger.info(u"Pujant un any màxim: %s" % from_date)
        else:
            logger.info(u"Última lectura trobada: %s" % last_measure)
            from_date = last_measure
        measures = c.GiscedataLecturesComptador.get_aggregated_measures(
            [comptador['id']], from_date
        )
        logger.info("S'han trobat %s mesures per pujar" % (
            len(measures)
        ))
        popper = Popper(measures)
        pops = popper.pop(bucket)
        while pops:
            j = push_amon_measures.delay(pops)
            logger.info("Job id:%s | %s/%s/%s" % (
                j.id, comptador['name'], len(pops), len(popper.items))
            )
            pops = popper.pop(bucket)


def enqueue_new_contracts(bucket=1, force=False):
    search_params = [
        ('etag', '=', False),
        ('state', 'not in', ('esborrany', 'validar', 'cancelada'))
    ]
    if not force:
        with setup_empowering_api() as em:
            items = em.contracts().get(sort="[('_updated', -1)]")['_items']
            if items:
                from_date = make_local_timestamp(items[0]['_updated'])
                search_params += [
                        '|',
                        ('create_date', '>', from_date),
                        ('write_date', '>', from_date)
                ]
    O = setup_peek()
    contracts_ids = O.GiscedataPolissa.search(search_params)
    logger.info('Found %s contracts to push', len(contracts_ids))
    popper = Popper(contracts_ids)
    pops = popper.pop(bucket)
    while pops:
        j = push_contracts.delay(pops)
        logger.info("Job id:%s" % j.id)
        pops = popper.pop(bucket)


def enqueue_contracts(contracts=None, force=False):
    O = setup_peek()
    # Busquem els que hem d'actualitzar
    if contracts is None:
        polisses_actives_ids = O.GiscedataPolissa.search([
            ('state', 'not in', ('esborrany', 'validar', 'cancelada')), ('contract_type', '=', '01')
        ])
        date_ago = (datetime.now() - relativedelta(months=3)).strftime('%Y-%m-%d')
        polisses_baixes_ids = O.GiscedataPolissa.search([
            ('state', '=', 'baixa'),
            ('data_baixa', '>=', date_ago)
        ], context={'active_test': False})
        polisses_ids = polisses_actives_ids + polisses_baixes_ids
    else:
        polisses_ids = O.GiscedataPolissa.search([
            ('name', 'in', contracts)
        ], context={'active_test': False})
    if not polisses_ids:
        logger.info('No contracts found')
        return
    fields_to_read = ['name', 'etag', 'comptadors', 'modcontractual_activa']
    if force:
        logger.info('Forcing pushing {} contracts'.format(len(polisses_ids)))
        for polissa_id in polisses_ids:
            push_contracts.delay([polissa_id])
        return
    for polissa in O.GiscedataPolissa.read(polisses_ids, fields_to_read):
        modcons = []
        is_new_contract = False
        try:
            with setup_empowering_api() as em:
                last_updated = em.contract(polissa['name']).get()['_updated']
                last_updated = make_local_timestamp(last_updated)
                etag_beedata = em.contract(polissa['name']).get()['_etag']
                # Update etag in ERP if it changed
                if etag_beedata != polissa['etag']:
                    logger.info('Contract {} changed "etag". Resynced "etag" from API.'.format(polissa['name']))
                    O.GiscedataPolissa.write(polissa['id'], {'etag': etag_beedata})
        except (libsaas.http.HTTPError, urllib2.HTTPError) as e:
            # A 404 is possible if we delete empowering contracts in insight engine
            # but keep etag in our database.
            # In this case we must force the re-upload as new contract
            if e.code != 404:
                raise e
            is_new_contract = True
            last_updated = '0'

        w_date = O.GiscedataPolissa.perm_read(polissa['id'])[0]['write_date']
        c_w_dates = []
        for comp_perm in O.GiscedataLecturesComptador.perm_read(polissa['comptadors']):
            c_w_dates.append(comp_perm['write_date'])
        c_w_date = max(c_w_dates)
        if w_date > last_updated and not is_new_contract:
            # Ara mirem quines modificaciones contractuals hem de pujar
            polissa = O.GiscedataPolissa.browse(polissa['id'])
            for modcon in polissa.modcontractuals_ids:
                perms = modcon.perm_read()
                if perms['write_date'] > last_updated:
                    logger.info('La modcontractual %s a actualitzar write_'
                                'date: %s last_update: %s' % (
                        modcon.name, perms['write_date'], last_updated))
                    modcons.append(modcon.id)
        elif c_w_date > last_updated and not is_new_contract:
            # Si no hi ha hagut canvis a les modificacions contractuals
            # però sí que s'ha tocat algun comptador fem una actualització
            # amb la última modificació contractual
            modcons.append(polissa['modcontractual_activa'][0])
        if modcons:
            logger.info('Polissa %s actualitzada a %s després de %s' % (
                polissa.name, w_date, last_updated))
            push_contracts.delay(polissa.id)
        if is_new_contract:
            logger.info("La polissa %s te etag pero ha estat borrada "
                        "d'empowering, es torna a pujar" % polissa['name'])
            push_contracts.delay([polissa['id']])


def enqueue_indexed(bucket=1, force=False, pricelist=False, wreport=False):
    """Busquem els grups indexats formats per llista preu + FEE que coincideixin
    Recuperem l'ultima data publicada per saber d'es d'on pujar
    Si l'agrupació no s'ha pujat mai, busquem factures i pujem desde la data més petita
    force = True: Força pujada
    wreport = True: Crea un report al director /tmp amb els preus mitjos horaris per agrupació
    """
    O = setup_peek()
    indexed_grouppeds = {}
    pids = O.GiscedataPolissa.search([('mode_facturacio', '=', 'index')])
    if wreport:
        import pandas as pd
        report_name = '/tmp/beedata_indexed_{}.xlsx'.format(datetime.now())
        writer = pd.ExcelWriter(report_name)
    for pol in O.GiscedataPolissa.read(pids, ['llista_preu', 'coeficient_d', 'coeficient_k', 'name', 'tarifa']):
        fee = pol['coeficient_d'] + pol['coeficient_k']
        llprice = pol['llista_preu'][1]
        # If pricelist name is specified, filter
        if pricelist:
            if pricelist not in llprice:
                continue
        tarifa = pol['tarifa'][1]
        key = (tarifa, '{} - {}'.format(llprice, fee))
        if key not in indexed_grouppeds:
            indexed_grouppeds[key] = [pol['id']]
        else:
            indexed_grouppeds[key].append(pol['id'])
    for group_key, contracts in indexed_grouppeds.items():
        # Search last indexed publish date
        tariff_id, cost = group_key
        pindexed_id = O.EmpoweringPriceIndexed.search(
            [('tariff_id', '=', str(tariff_id)), ('tariff_cost_id', '=', str(cost))]
        )
        if pindexed_id and not force:
            ldate = O.EmpoweringPriceIndexed.read(
                pindexed_id[0], ['empowering_price_indexed_last_push']
            )['empowering_price_indexed_last_push']
            logger.info('Grup indexats %s, pujem desde ultima data %s', group_key, ldate)
        else:
            # Si no hem pujat mai l'agrupacio, busquem el minim per di lectura factura
            ffids = O.GiscedataFacturacioFactura.search([
                ('polissa_id', 'in', contracts),
                ('type', '=', 'out_invoice'),
                ('llista_preu.name', '=like', '%ndex%')
            ])
            if ffids:
                ldate = min([x['data_inici'] for x in O.GiscedataFacturacioFactura.read(ffids, ['data_inici'])])
            else:
                # Agrupacio encara sense factures o contractes en esborrany
                continue
            if force:
                logger.info('Grup indexats %s FORÇATS, pujem desde primera factura %s', group_key, ldate)
            else:
                logger.info('Grup indexats %s NO PUJAT MAI ENCARA, pujem desde primera factura %s', group_key, ldate)
        fact_ids = []
        ldate = ldate[:10]
        for pol_id in contracts:
            pol_fact_ids = O.GiscedataFacturacioFactura.search([
                ('polissa_id', '=', pol_id),
                ('data_inici', '>', ldate),
                ('type', '=', 'out_invoice')
            ])
            for f_id in pol_fact_ids:
                f_data_fi = O.GiscedataFacturacioFactura.read(f_id, ['data_final'])['data_final']
                pol_mode_facturacio = O.GiscedataPolissa.read(pol_id, ['mode_facturacio'],
                                                              context={'date': f_data_fi})['mode_facturacio']
                if pol_mode_facturacio == 'index':
                    fact_ids.append(f_id)
        if fact_ids:
            logger.info('Pujant %s grup indexats amb #facts %s', group_key, len(fact_ids))
            r = push_indexeds.delay((group_key, fact_ids))
            if wreport:
                at, ak = group_key
                ak = ak.replace(' - ', '')
                if not r:
                    continue
                #todo: get price from responses
                df = pd.DataFrame(eval(r))
                df.to_excel(writer, sheet_name='{}{}'.format(at, ak))
    if wreport:
        logger.info('Report guardat: %s', report_name)
        writer.save()

@job(setup_queue(name='measures'), connection=setup_redis(), timeout=3600)
@sentry.capture_exceptions
def push_amon_measures(measures):
    """Pugem les mesures a l'Insight Engine
    """
    logging.basicConfig(level=logging.INFO)
    with setup_empowering_api() as em:
        c = setup_peek()
        amon = AmonConverter(c)
        start = datetime.now()
        measures_to_push = amon.aggregated_measures_to_amon(measures)
        logger.info("Enviant de %s (id:%s) a %s (id:%s)" % (
            measures[-1]['timestamp'], measures[-1]['meter_id'],
            measures[0]['timestamp'], measures[0]['meter_id']
        ))
        stop = datetime.now()
        logger.info('Mesures transformades en %s' % (stop - start))
        start = datetime.now()
        # Check which endpoint to use
        residential = measures_to_push.get('R')
        if residential:
            logger.debug('Pushing %s', residential)
            em.residential_timeofuse_amon_measures().create(residential)
        tertiary = measures_to_push.get('T')
        if tertiary:
            logger.debug('Pushing %s', residential)
            em.tertiary_amon_measures().create(tertiary)
        # Save last timestamp
        last_measure = measures[0]
        c.GiscedataLecturesComptador.update_empowering_last_measure(
            [last_measure['meter_id']], last_measure['timestamp']
        )
        stop = datetime.now()
        logger.info('Mesures enviades en %s' % (stop - start))
        logger.info("%s measures creades" % len(measures))


@job(setup_queue(name='profiles'), connection=setup_redis(), timeout=3600)
@sentry.capture_exceptions
def push_amon_profiles(profiles, collection):
    """Pugem les mesures a l'Insight Engine
    """
    with setup_empowering_api() as em:
        c = setup_peek()
        amon = AmonConverter(c)
        measures_to_push = amon.profiles_to_amon(profiles, collection)
        for cups, m_to_push in measures_to_push.items():
            em.amon_measures().create(m_to_push)
            last_measure = max(
                make_local_timestamp(x['timestamp'])
                for x in m_to_push['measurements']
            )
            pol_id = c.GiscedataPolissa.search([
                ('cups.name', 'like', cups),
                ('state', 'not in', ('esborrany', 'validar', 'cancelada', 'baixa')),
                ('data_alta', '<=', last_measure),
                '|',
                ('data_baixa', '>=', last_measure),
                ('data_baixa', '=', False)
            ], context={'active_test': False})
            if not pol_id:
                continue
            if len(pol_id) > 1:
                raise Exception('{} contracts found! CUPS: {}. Last measure: {}'.format(len(pol_id), cups, last_measure))
            pol = c.GiscedataPolissa.read(pol_id[0], ['name', 'empowering_last_profile_measure'])
            if last_measure > pol['empowering_last_profile_measure']:
                logger.info('Updating polissa (id: %s) to last measure: %s', pol['name'], last_measure)
                c.GiscedataPolissa.write(pol_id, {
                    'empowering_last_profile_measure': last_measure
                })


@job(setup_queue(name='contracts'), connection=setup_redis(), timeout=3600)
@sentry.capture_exceptions
def push_modcontracts(modcons, etag):
    """modcons is a list of modcons to push
    """
    em = setup_empowering_api()
    O = setup_peek()
    amon = AmonConverter(O)
    fields_to_read = ['data_inici', 'polissa_id']
    modcons = O.GiscedataPolissaModcontractual.read(modcons, fields_to_read)
    modcons = sorted_by_key(modcons, 'data_inici')
    for modcon in modcons:
        amon_data = amon.contract_to_amon(
            modcon['polissa_id'][0],
            {'modcon_id': modcon['id']}
        )[0]
        response = em.contract(modcon['polissa_id'][1]).update(amon_data, etag)
        if check_response(response, amon_data):
            etag = response['_etag']
    O.GiscedataPolissa.write(modcon['polissa_id'][0], {'etag': etag})


@job(setup_queue(name='contracts'), connection=setup_redis(), timeout=3600)
@sentry.capture_exceptions
def push_contracts(contracts_id):
    """Pugem els contractes
    """
    import logging
    logging.basicConfig(level=logging.INFO)
    with setup_empowering_api() as em:
        O = setup_peek()
        amon = AmonConverter(O)
        if not isinstance(contracts_id, (list, tuple)):
            contracts_id = [contracts_id]
        for pol in O.GiscedataPolissa.read(contracts_id, ['name', 'etag']):
            amon_data = amon.contract_to_amon(pol['id'])[0]
            logger.debug('Contract data %s', amon_data)
            try:
                if pol['etag']:
                    response = em.contract(pol['name']).update(
                        amon_data, pol['etag']
                    )
                else:
                    response = em.contracts().create(amon_data)
            except urllib2.HTTPError as err:
                raise Exception('HTTPError code {}. Error: {}'.format(err.code, err.read())) 
            O.GiscedataPolissa.write([pol['id']], {'etag': response['_etag']})


@job(setup_queue(name='tariffs'), connection=setup_redis(), timeout=3600)
@sentry.capture_exceptions
def push_tariffs(tariffs):
    c = setup_peek()
    a = AmonConverter(c)
    result = a.tariff_to_amon(*tariffs)
    with setup_empowering_api() as em:
        for r in result:
            try:
                print(r)
                em.tariffs().create(r)
            except urllib2.HTTPError as err:
                print(err.read())
                raise


@job(setup_queue(name='indexeds'), connection=setup_redis(), timeout=3600)
@sentry.capture_exceptions
def push_indexeds(indexeds):
    """Preus horaris indexats agrupats per llista de preu i FEE
    """
    c = setup_peek()
    a = AmonConverter(c)
    result = a.indexed_to_amon(*indexeds)
    with setup_empowering_api() as em:
        try:
            response = em.price_indexed().create(result)
            if response['_status'] == 'OK':
                msg_ok = 'Grup indexats PUJAT CORRECTAMENT! %s', response
                print(msg_ok)
                logger.info(msg_ok)
                # If a list is POSTed it will return an ordered list with documents
                # with eve fields added
                for item in response['_items']:
                    etag = item['_etag']
                    # First or last same tariffId and cost
                    tid = result[0]['tariffId']
                    cid = result[0]['tariffCostId']
                    epid = c.EmpoweringPriceIndexed.search([
                        ('tariff_id', '=', tid), ('tariff_cost_id', '=', cid)
                    ])
                    ldate = max([x['datetime'] for x in result])
                    if epid:
                        c.EmpoweringPriceIndexed.write(epid, {
                            'empowering_price_indexed_last_push': ldate,
                            'etag': etag
                        })
                    else:
                        c.EmpoweringPriceIndexed.create({
                            'tariff_id': tid,
                            'tariff_cost_id': cid,
                            'empowering_price_indexed_last_push': ldate,
                            'etag': etag
                        })
        except urllib2.HTTPError as err:
            print(err.read())
            raise
